/*----------------------------------------------------------------------------*/
/** \file		CWUH20K1.cpp
 *
 *  C++ plugin code for X-Fi What-You-Hear. 
 *
 * @author      Robert Jan Ridder
 * 
 * $Date: 2005/05/03 02:10:32 $
 * $Revision: 1.42 $
 * 
 *****************************************************************************
 * Copyright (C) Creative Technology, Ltd., 2004. All rights reserved.
 *****************************************************************************
 */
/** \mainpage
 *  XFi What-You-Hear routines.
 *
 *  The What-You-Hear module is implemented with Audio Ring resources only (no DSP code).
 *  The structure is as follows:
 * 
 *
 * \verbatim
 *      DUMMY SCALER       SCALER  
 *          1.0          recording_level
 * 
 *          |\              |\
 *  FL o----| +-----+-------| +-----------------------o FL out Multich. Rec
 *          |/      |       |/  
 *             [0]  |         [0]
 *                  |             +------+
 *                  +-------------+ SEND +----+
 *                                +------+    |
 *                                   [0]      :
 *          |\              |\
 *  FR o----| +-----+-------| +-----------------------o FR out Multich. Rec
 *          |/      |       |/  
 *             [1]  |         [1]
 *                  |             +------+
 *                  +-------------+ SEND +--------+
 *                                +------+        |
 *                                   [1]          :
 *          |\              |\
 *  RL o----| +-----+-------| +-----------------------o FL out Multich. Rec
 *          |/      |       |/  
 *             [2]  |         [2]
 *                  |             +------+
 *                  +-------------+ SEND +----+
 *                                +------+    |
 *                                   [2]      :
 *          |\              |\
 *  RR o----| +-----+-------| +-----------------------o FR out Multich. Rec
 *          |/      |       |/  
 *             [3]  |         [3]
 *                  |             +------+
 *                  +-------------+ SEND +--------+
 *                                +------+        |
 *                                   [3]          :
 *          |\              |\
 *  FC o----| +-----+-------| +-----------------------o FC out Multich. Rec
 *          |/      |       |/  
 *             [4]  |         [4]
 *                  |             +------+
 *                  +-------------+ SEND +----+
 *                  |             +------+    |
 *                  |                [4]      :
 *                  |             +------+
 *                  +-------------+ SEND +--------+
 *                                +------+        |
 *                                   [8]          :
 *          |\              |\
 *  SL o----| +-----+-------| +-----------------------o SL out Multich. Rec
 *          |/      |       |/  
 *             [5]  |         [5]
 *                  |             +------+
 *                  +-------------+ SEND +----+
 *                  |             +------+    |
 *                  |                [5]      :
 *                  |             +------+
 *                  +-------------+ SEND +--------+
 *                                +------+        |
 *                                   [6*]         :
 *          |\              |\
 *  SR o----| +-----+-------| +-----------------------o SR out Multich. Rec
 *          |/      |       |/  
 *             [6]  |         [6]
 *                  |             +------+
 *                  +-------------+ SEND +--------+
 *                                +------+        |
 *                                   [6*]         :
 * 
 *          |\              |\
 * Sub o----| +-----+-------| +-----------------------o Subwoofer out Multich. Rec
 *          |/      |       |/  
 *             [7]  |         [7]
 *                  |             +------+
 *                  +-------------+ SEND +----+
 *                  |             +------+    |
 *                  |                [7]      :
 *                  |             +------+
 *                  +-------------+ SEND +--------+
 *                                +------+        |
 *                                   [9]          :
 *                                            :
 *                                            |
 *                                            |
 *                                            |       +--------+
 *                                        [0] +-------+        |
 *                                        [2] --------+        |
 *                                        [4] --------+  SUM L +--------o Stereo L Rec
 *                                        [5] --------+   [0]  |
 *                                        [7] --------+        |
 *                                                    +--------+
 * 
 *                                                :
 *                                                |
 *                                                |
 *                                                |   +--------+
 *                                        [1]     +---+        |
 *                                        [3]     ----+        |
 *                                        [8]     ----+  SUM R +--------o Stereo R Rec
 *                                        [6]     ----+   [1]  |
 *                                        [9]     ----+        |
 *                                                    +--------+
 * 
 * \endverbatim
 *
 * The dummy scalers at the input are required to provide a connection point for the
 * send nodes going into the summation nodes for the stereo recording stream.
 * The Center and LFE channels are added to both channels in the stereo mix with 3 dB attenuation.
 * As an exception, for 6.1 mode, the rear center is added in the same way.
 *
 */

#include "CWUH20K1.h"


/******************************************************************************
 * CWUH20K1::CWUH20K1()
 ******************************************************************************/
/**
 * @brief   Constructor of this plugin class. Called from the driver.
 *
 */
CWUH20K1::CWUH20K1() : C20KxPlugin( NULL, 
									   NULL,
									   NULL,      //no outer
									   0,
									   NULL )
{
	int ch_i;

	for ( ch_i = 0;  ch_i < 2;  ++ch_i )
	{
		m_pBusNode[ch_i] = NULL;
	}

	for ( ch_i = 0;  ch_i < MCH_MAX+2;  ++ch_i )
	{
		m_pSendNode[ch_i]   = NULL;
	}

	for ( ch_i = 0;  ch_i < MCH_MAX;  ++ch_i )
	{
		m_pDummyNode[ch_i] = NULL;
		m_pScalerNode[ch_i] = NULL;
	}
}


/******************************************************************************
 * CWUH20K1::~CWUH20K1()
 ******************************************************************************/
/**
 * @brief   Destructor of this plugin class. Called from the driver.
 *
 */
CWUH20K1::~CWUH20K1()
{
	int ch_i;
	CRM_CONNECT_STATUS foo;
	for ( ch_i = 0;  ch_i < 2;  ++ch_i )
	{
		if (m_pBusNode[ch_i])
		{
			m_pBusNode[ch_i]->ConnectionRelease(&foo, 0);
			m_pBusNode[ch_i]->Release();
		}
	}
	
	for ( ch_i = 0;  ch_i < MCH_MAX+2;  ++ch_i )
	{
		if (m_pSendNode[ch_i])			m_pSendNode[ch_i]->Release();
	}

	for ( ch_i = 0;  ch_i < MCH_MAX;  ++ch_i )
	{
		if (m_pDummyNode[ch_i])  		m_pDummyNode[ch_i]->Release();
		if (m_pScalerNode[ch_i])		m_pScalerNode[ch_i]->Release();
	}
}


/******************************************************************************
 * CWUH20K1::rampScalerNodeLevel( I20KxPeAudioScalerNode *pScalerNode, CTFLOAT dBlevel )
 ******************************************************************************/
/**
 * @brief   Calculate and ramp level of multichannel recording
 *
 * @param pScalerNode		Scaler node to affect
 * @param dBlevel           Level (dB)
 */
HRESULT CWUH20K1::rampScalerNodeLevel( I20KxPeAudioScalerNode *pScalerNode, float dBlevel )
{
	return S_OK;
}


/******************************************************************************
 * CWUH20K1::rampScalerNodeLevels( float dBlevel )
 ******************************************************************************/
/**
 * @brief   Calculate and set level of multichannel recording
 *
 * @param dBlevel	Recording level (dB)
 */
HRESULT CWUH20K1::rampScalerNodeLevels( float dBlevel )
{
	HRESULT hr;  CTDWORD ch_i;

	for ( ch_i = 0;  ch_i < MCH_MAX;  ++ch_i )
	{
 		if ( FAILED(hr = rampScalerNodeLevel( m_pScalerNode[ch_i], dBlevel )) )
		{
			break;
		}
	}

	return hr;
}


/******************************************************************************
 * CWUH20K1::setParamValue(CTDWORD dwParam, PARAM_VALUE value)
 ******************************************************************************/
/**
 * @brief   Set the value of the specified parameter. Called from the driver.
 *
 * @param dwParam
 * Index of the high-level parameter
 * @param value
 * Value to set the parameter to.
 */
STDMETHODIMP CWUH20K1::setParamValue(CTDWORD dwParam, PARAM_VALUE value)
{
	HRESULT hr;

	switch (dwParam)
 	{
 	case 0:		/* recording level */
		hr = rampScalerNodeLevels( value.floatVal );
		break;
	default:
		hr = S_OK;
		break;
	}
	return hr;
}
